@@ -247,6 +247,7 @@ class Opml extends BaseController {
         $this->writer->writeAttributeNS('selfoss', 'params', null, html_entity_decode($source['params']));
 
         $this->writer->endElement();  // outline
+        \F3::get('logger')->log("done exporting source ".$source['title'], \DEBUG);
     }
 
 
@@ -259,6 +260,7 @@ class Opml extends BaseController {
         $this->sourcesDao = new \daos\Sources();
         $this->tagsDao = new \daos\Tags();
 
+        \F3::get('logger')->log('start OPML export', \DEBUG);
         $this->writer = new \XMLWriter();
         $this->writer->openMemory();
         $this->writer->setIndent(1);
@@ -276,11 +278,13 @@ class Opml extends BaseController {
         $this->writer->writeAttribute('version', '1.0');
         $this->writer->writeAttribute('createdOn', date('r'));
         $this->writer->endElement();  // meta
+        \F3::get('logger')->log('OPML export: finished writing meta', \DEBUG);
 
         $this->writer->startElement('head');
         $user = \F3::get('username');
         $this->writer->writeElement('title', ($user?$user.'\'s':'My') . ' subscriptions in selfoss');
         $this->writer->endElement();  // head
+        \F3::get('logger')->log('OPML export: finished writing head', \DEBUG);
 
         $this->writer->startElement('body');
 
@@ -298,11 +302,13 @@ class Opml extends BaseController {
         // create associative array with tag names as keys, colors as values
         $tagColors = array();
         foreach ($this->tagsDao->get() as $key => $tag) {
+            \F3::get('logger')->log("OPML export: tag ".$tag['tag']." has color ".$tag['color'], \DEBUG);
             $tagColors[$tag['tag']] = $tag['color'];
         }
 
         // generate outline elements for all sources
         foreach ($sources['tagged'] as $tag => $children) {
+            \F3::get('logger')->log("OPML export: exporting tag $tag sources", \DEBUG);
             $this->writer->startElement('outline');
             $this->writer->writeAttribute('title', $tag);
             $this->writer->writeAttribute('text', $tag);
@@ -315,6 +321,8 @@ class Opml extends BaseController {
 
             $this->writer->endElement();  // outline
         }
+
+        \F3::get('logger')->log("OPML export: exporting untagged sources", \DEBUG);
         foreach ($sources['untagged'] as $key => $source) {
             $this->writeSource($source);
         }
@@ -322,9 +330,10 @@ class Opml extends BaseController {
         $this->writer->endElement();  // body
 
         $this->writer->endDocument();
+        \F3::get('logger')->log('finished OPML export', \DEBUG);
 
         // save content as file and suggest file name
-        header('Content-Disposition: attachment; filename="selfoss-subscriptions.opml"');
+        header('Content-Disposition: attachment; filename="selfoss-subscriptions.xml"');
         header('Content-Type: text/xml; charset=UTF-8');
         echo $this->writer->outputMemory();
     }
