@@ -186,24 +186,30 @@ class Auth {
 	*	HTTP basic auth mechanism
 	*	@return bool
 	*	@param $func callback
-	*	@param $halt bool
 	**/
-	function basic($func=NULL,$halt=TRUE) {
+	function basic($func=NULL) {
 		$fw=Base::instance();
 		$realm=$fw->get('REALM');
+		$hdr=NULL;
+		if (isset($_SERVER['HTTP_AUTHORIZATION']))
+			$hdr=$_SERVER['HTTP_AUTHORIZATION'];
+		elseif (isset($_SERVER['REDIRECT_HTTP_AUTHORIZATION']))
+			$hdr=$_SERVER['REDIRECT_HTTP_AUTHORIZATION'];
+		if (!empty($hdr))
+			list($_SERVER['PHP_AUTH_USER'],$_SERVER['PHP_AUTH_PW'])=
+				explode(':',base64_decode(substr($hdr,6)));
 		if (isset($_SERVER['PHP_AUTH_USER'],$_SERVER['PHP_AUTH_PW']) &&
 			$this->login(
 				$_SERVER['PHP_AUTH_USER'],
 				$func?
-					$func($_SERVER['PHP_AUTH_PW']):
+					$fw->call($func,$_SERVER['PHP_AUTH_PW']):
 					$_SERVER['PHP_AUTH_PW'],
 				$realm
 			))
 			return TRUE;
 		if (PHP_SAPI!='cli')
 			header('WWW-Authenticate: Basic realm="'.$realm.'"');
-		if ($halt)
-			$fw->error(401);
+		$fw->status(401);
 		return FALSE;
 	}
 
