@@ -27,7 +27,9 @@ abstract class Cursor extends \Magic {
 		//! Query results
 		$query=array(),
 		//! Current position
-		$ptr=0;
+		$ptr=0,
+		//! Event listeners
+		$trigger=array();
 
 	/**
 	*	Return records (array of mapper objects) that match criteria
@@ -70,8 +72,8 @@ abstract class Cursor extends \Magic {
 
 	/**
 	*	Return array containing subset of records matching criteria,
-	*	total number of records in superset, number of subsets available,
-	*	and actual subset position
+	*	total number of records in superset, specified limit, number of
+	*	subsets available, and actual subset position
 	*	@return array
 	*	@param $pos int
 	*	@param $size int
@@ -79,7 +81,7 @@ abstract class Cursor extends \Magic {
 	*	@param $options array
 	**/
 	function paginate($pos=0,$size=10,$filter=NULL,array $options=NULL) {
-		$total=$this->count($filter,$options);
+		$total=$this->count($filter);
 		$count=ceil($total/$size);
 		$pos=max(0,min($pos,$count-1));
 		return array(
@@ -90,6 +92,7 @@ abstract class Cursor extends \Magic {
 				)
 			),
 			'total'=>$total,
+			'limit'=>$size,
 			'count'=>$count,
 			'pos'=>$pos<$count?$pos:0
 		);
@@ -107,19 +110,19 @@ abstract class Cursor extends \Magic {
 	}
 
 	/**
-	*	Move pointer to first record in cursor
+	*	Map to first record in cursor
 	*	@return mixed
 	**/
 	function first() {
-		return $this->query[$this->ptr=0];
+		return $this->skip(-$this->ptr);
 	}
 
 	/**
-	*	Move pointer to last record in cursor
+	*	Map to last record in cursor
 	*	@return mixed
 	**/
 	function last() {
-		return $this->query[$this->ptr=($ctr=count($this->query))?$ctr-1:0];
+		return $this->skip(($ofs=count($this->query)-$this->ptr)?$ofs-1:0);
 	}
 
 	/**
@@ -167,6 +170,38 @@ abstract class Cursor extends \Magic {
 		$this->ptr=0;
 	}
 
+	/**
+	*	Define onload trigger
+	*	@return closure
+	**/
+	function onload($func) {
+		return $this->trigger['load']=$func;
+	}
+
+	/**
+	*	Define oninsert trigger
+	*	@return closure
+	**/
+	function oninsert($func) {
+		return $this->trigger['insert']=$func;
+	}
+
+	/**
+	*	Define onupdate trigger
+	*	@return closure
+	**/
+	function onupdate($func) {
+		return $this->trigger['update']=$func;
+	}
+
+	/**
+	*	Define onerase trigger
+	*	@return closure
+	**/
+	function onerase($func) {
+		return $this->trigger['erase']=$func;
+	}
+
 	/**
 	*	Reset cursor
 	*	@return NULL
