@@ -29,6 +29,8 @@ class Mapper extends \DB\Cursor {
 		//! Database engine
 		$engine,
 		//! SQL table
+		$source,
+		//! SQL table (quoted)
 		$table,
 		//! Last insert ID
 		$_id,
@@ -54,8 +56,8 @@ class Mapper extends \DB\Cursor {
 	**/
 	function set($key,$val) {
 		if (array_key_exists($key,$this->fields)) {
-			if (!is_null($val) || !$this->fields[$key]['nullable'])
-				$val=$this->value($this->fields[$key]['pdo_type'],$val);
+			$val=is_null($val) && $this->fields[$key]['nullable']?
+				NULL:$this->value($this->fields[$key]['pdo_type'],$val);
 			if ($this->fields[$key]['value']!==$val ||
 				$this->fields[$key]['default']!==$val)
 				$this->fields[$key]['changed']=TRUE;
@@ -137,12 +139,19 @@ class Mapper extends \DB\Cursor {
 		$mapper=clone($this);
 		$mapper->reset();
 		foreach ($row as $key=>$val) {
-			$var=array_key_exists($key,$this->fields)?'fields':'adhoc';
+			if (array_key_exists($key,$this->fields))
+				$var='fields';
+			elseif (array_key_exists($key,$this->adhoc))
+				$var='adhoc';
+			else
+				continue;
 			$mapper->{$var}[$key]['value']=$val;
 			if ($var=='fields' && $mapper->{$var}[$key]['pkey'])
 				$mapper->{$var}[$key]['previous']=$val;
 		}
 		$mapper->query=array(clone($mapper));
+		if (isset($mapper->trigger['load']))
+			\Base::instance()->call($mapper->trigger['load'],$mapper);
 		return $mapper;
 	}
 
@@ -156,7 +165,7 @@ class Mapper extends \DB\Cursor {
 			$obj=$this;
 		return array_map(
 			function($row) {
-				return $row['value' ];
+				return $row['value'];
 			},
 			$obj->fields+$obj->adhoc
 		);
@@ -192,14 +201,25 @@ class Mapper extends \DB\Cursor {
 			$sql.=' WHERE '.$filter;
 		}
 		if ($options['group'])
-			$sql.=' GROUP BY '.$options['group'];
-		if ($options['order'])
-			$sql.=' ORDER BY '.$options['order'];
+			$sql.=' GROUP BY '.implode(',',array_map(
+				array($this->db,'quotekey'),
+				explode(',',$options['group'])));
+		if ($options['order']) {
+			$db=$this->db;
+			$sql.=' ORDER BY '.implode(',',array_map(
+				function($str) use($db) {
+					return preg_match('/(\w+)(?:\h+(ASC|DESC))?/i',
+						$str,$parts)?
+						($db->quotekey($parts[1]).
+						(isset($parts[2])?(' '.$parts[2]):'')):$str;
+				},
+				explode(',',$options['order'])));
+		}
 		if ($options['limit'])
-			$sql.=' LIMIT '.$options['limit'];
+			$sql.=' LIMIT '.(int)$options['limit'];
 		if ($options['offset'])
-			$sql.=' OFFSET '.$options['offset'];
-		$result=$this->db->exec($sql.';',$args,$ttl);
+			$sql.=' OFFSET '.(int)$options['offset'];
+		$result=$this->db->exec($sql,$args,$ttl);
 		$out=array();
 		foreach ($result as &$row) {
 			foreach ($row as $field=>&$val) {
@@ -237,16 +257,20 @@ class Mapper extends \DB\Cursor {
 		$adhoc='';
 		foreach ($this->adhoc as $key=>$field)
 			$adhoc.=','.$field['expr'].' AS '.$this->db->quotekey($key);
-		return $this->select('*'.$adhoc,$filter,$options,$ttl);
+		return $this->select(implode(',',
+			array_map(array($this->db,'quotekey'),array_keys($this->fields))).
+			$adhoc,$filter,$options,$ttl);
 	}
 
 	/**
 	*	Count records that match criteria
 	*	@return int
 	*	@param $filter string|array
+	*	@param $ttl int
 	**/
-	function count($filter=NULL) {
-		$sql='SELECT COUNT(*) AS rows FROM '.$this->table;
+	function count($filter=NULL,$ttl=0) {
+		$sql='SELECT COUNT(*) AS '.
+			$this->db->quotekey('rows').' FROM '.$this->table;
 		$args=array();
 		if ($filter) {
 			if (is_array($filter)) {
@@ -258,7 +282,7 @@ class Mapper extends \DB\Cursor {
 			}
 			$sql.=' WHERE '.$filter;
 		}
-		$result=$this->db->exec($sql.';',$args);
+		$result=$this->db->exec($sql,$args,$ttl);
 		return $result[0]['rows'];
 	}
 
@@ -269,25 +293,27 @@ class Mapper extends \DB\Cursor {
 	*	@param $ofs int
 	**/
 	function skip($ofs=1) {
-		if ($out=parent::skip($ofs)) {
-			foreach ($this->fields as $key=>&$field) {
-				$field['value']=$out->fields[$key]['value'];
-				$field['changed']=FALSE;
-				if ($field['pkey'])
-					$field['previous']=$out->fields[$key]['value'];
-				unset($field);
-			}
-			foreach ($this->adhoc as $key=>&$field) {
-				$field['value']=$out->adhoc[$key]['value'];
-				unset($field);
-			}
+		$out=parent::skip($ofs);
+		$dry=$this->dry();
+		foreach ($this->fields as $key=>&$field) {
+			$field['value']=$dry?NULL:$out->fields[$key]['value'];
+			$field['changed']=FALSE;
+			if ($field['pkey'])
+				$field['previous']=$dry?NULL:$out->fields[$key]['value'];
+			unset($field);
 		}
+		foreach ($this->adhoc as $key=>&$field) {
+			$field['value']=$dry?NULL:$out->adhoc[$key]['value'];
+			unset($field);
+		}
+		if (isset($this->trigger['load']))
+			\Base::instance()->call($this->trigger['load'],$this);
 		return $out;
 	}
 
 	/**
 	*	Insert new record
-	*	@return array
+	*	@return object
 	**/
 	function insert() {
 		$args=array();
@@ -298,7 +324,7 @@ class Mapper extends \DB\Cursor {
 		$inc=NULL;
 		foreach ($this->fields as $key=>&$field) {
 			if ($field['pkey']) {
-				$pkeys[]=$key;
+				$pkeys[$key]=$field['previous'];
 				$field['previous']=$field['value'];
 				if (!$inc && $field['pdo_type']==\PDO::PARAM_INT &&
 					empty($field['value']) && !$field['nullable'])
@@ -316,31 +342,39 @@ class Mapper extends \DB\Cursor {
 		if ($fields)
 			$this->db->exec(
 				'INSERT INTO '.$this->table.' ('.$fields.') '.
-				'VALUES ('.$values.');',$args
+				'VALUES ('.$values.')',$args
 			);
 		$seq=NULL;
-		if ($this->engine=='pgsql')
-			$seq=$this->table.'_'.end($pkeys).'_seq';
-		$this->_id=$this->db->lastinsertid($seq);
+		if ($this->engine=='pgsql') {
+			$names=array_keys($pkeys);
+			$seq=$this->source.'_'.end($names).'_seq';
+		}
+		if ($this->engine!='oci')
+			$this->_id=$this->db->lastinsertid($seq);
 		if (!$inc) {
 			$ctr=0;
 			$query='';
 			$args='';
-			foreach ($pkeys as $pkey) {
+			foreach (array_keys($pkeys) as $pkey) {
 				$query.=($query?' AND ':'').$this->db->quotekey($pkey).'=?';
 				$args[$ctr+1]=$this->fields[$pkey]['value'];
 				$ctr++;
 			}
-			return $this->load(array($query,$args));
+			$out=$query?$this->load(array($query,$args)):$this;
 		}
-		// Reload to obtain default and auto-increment field values
-		return $this->load(array($inc.'=?',
-			$this->value($this->fields[$inc]['pdo_type'],$this->_id)));
+		else
+			// Reload to obtain default and auto-increment field values
+			$out=$this->load(array($inc.'=?',
+				$this->value($this->fields[$inc]['pdo_type'],$this->_id)));
+		if (isset($this->trigger['insert']))
+			\Base::instance()->call($this->trigger['insert'],
+				array($this,$pkeys));
+		return $out;
 	}
 
 	/**
 	*	Update current record
-	*	@return array
+	*	@return object
 	**/
 	function update() {
 		$args=array();
@@ -353,18 +387,24 @@ class Mapper extends \DB\Cursor {
 				$args[$ctr+1]=array($field['value'],$field['pdo_type']);
 				$ctr++;
 			}
+		$pkeys=array();
 		foreach ($this->fields as $key=>$field)
 			if ($field['pkey']) {
 				$filter.=($filter?' AND ':'').$this->db->quotekey($key).'=?';
 				$args[$ctr+1]=array($field['previous'],$field['pdo_type']);
+				$pkeys[$key]=$field['previous'];
 				$ctr++;
 			}
 		if ($pairs) {
 			$sql='UPDATE '.$this->table.' SET '.$pairs;
 			if ($filter)
 				$sql.=' WHERE '.$filter;
-			return $this->db->exec($sql.';',$args);
+			$this->db->exec($sql,$args);
+			if (isset($this->trigger['update']))
+				\Base::instance()->call($this->trigger['update'],
+					array($this,$pkeys));
 		}
+		return $this;
 	}
 
 	/**
@@ -388,10 +428,12 @@ class Mapper extends \DB\Cursor {
 		$args=array();
 		$ctr=0;
 		$filter='';
+		$pkeys=array();
 		foreach ($this->fields as $key=>&$field) {
 			if ($field['pkey']) {
 				$filter.=($filter?' AND ':'').$this->db->quotekey($key).'=?';
 				$args[$ctr+1]=array($field['previous'],$field['pdo_type']);
+				$pkeys[$key]=$field['previous'];
 				$ctr++;
 			}
 			$field['value']=NULL;
@@ -406,8 +448,12 @@ class Mapper extends \DB\Cursor {
 		}
 		parent::erase();
 		$this->skip(0);
-		return $this->db->
+		$out=$this->db->
 			exec('DELETE FROM '.$this->table.' WHERE '.$filter.';',$args);
+		if (isset($this->trigger['erase']))
+			\Base::instance()->call($this->trigger['erase'],
+				array($this,$pkeys));
+		return $out;
 	}
 
 	/**
@@ -433,9 +479,13 @@ class Mapper extends \DB\Cursor {
 	*	Hydrate mapper object using hive array variable
 	*	@return NULL
 	*	@param $key string
+	*	@param $func callback
 	**/
-	function copyfrom($key) {
-		foreach (\Base::instance()->get($key) as $key=>$val)
+	function copyfrom($key,$func=NULL) {
+		$var=\Base::instance()->get($key);
+		if ($func)
+			$var=$func($var);
+		foreach ($var as $key=>$val)
 			if (in_array($key,array_keys($this->fields))) {
 				$field=&$this->fields[$key];
 				if ($field['value']!==$val) {
@@ -465,17 +515,30 @@ class Mapper extends \DB\Cursor {
 		return $this->fields;
 	}
 
+	/**
+	*	Return field names
+	*	@return array
+	*	@param $adhoc bool
+	**/
+	function fields($adhoc=TRUE) {
+		return array_keys($this->fields+($adhoc?$this->adhoc:array()));
+	}
+
 	/**
 	*	Instantiate class
 	*	@param $db object
 	*	@param $table string
+	*	@param $fields array|string
 	*	@param $ttl int
 	**/
-	function __construct(\DB\SQL $db,$table,$ttl=60) {
+	function __construct(\DB\SQL $db,$table,$fields=NULL,$ttl=60) {
 		$this->db=$db;
 		$this->engine=$db->driver();
+		if ($this->engine=='oci')
+			$table=strtoupper($table);
+		$this->source=$table;
 		$this->table=$this->db->quotekey($table);
-		$this->fields=$db->schema($table,$ttl);
+		$this->fields=$db->schema($table,$fields,$ttl);
 		$this->reset();
 	}
 
