@@ -54,15 +54,25 @@ class Session extends Mapper {
 	**/
 	function write($id,$data) {
 		$fw=\Base::instance();
+		$sent=headers_sent();
 		$headers=$fw->get('HEADERS');
 		$this->load(array('session_id=?',$id));
+		$csrf=$fw->hash($fw->get('ROOT').$fw->get('BASE')).'.'.
+			$fw->hash(mt_rand());
 		$this->set('session_id',$id);
 		$this->set('data',$data);
+		$this->set('csrf',$sent?$this->csrf():$csrf);
 		$this->set('ip',$fw->get('IP'));
 		$this->set('agent',
 			isset($headers['User-Agent'])?$headers['User-Agent']:'');
 		$this->set('stamp',time());
 		$this->save();
+		if (!$sent) {
+			if ($_COOKIE['_'])
+				setcookie('_','',strtotime('-1 year'));
+			call_user_func_array('setcookie',
+				array('_',$csrf)+$fw->get('JAR'));
+		}
 		return TRUE;
 	}
 
@@ -73,6 +83,9 @@ class Session extends Mapper {
 	**/
 	function destroy($id) {
 		$this->erase(array('session_id=?',$id));
+		setcookie(session_name(),'',strtotime('-1 year'));
+		unset($_COOKIE[session_name()]);
+		header_remove('Set-Cookie');
 		return TRUE;
 	}
 
@@ -86,6 +99,16 @@ class Session extends Mapper {
 		return TRUE;
 	}
 
+	/**
+	*	Return anti-CSRF tokan associated with specified session ID
+	*	@return string|FALSE
+	*	@param $id string
+	**/
+	function csrf($id=NULL) {
+		$this->load(array('session_id=?',$id?:session_id()));
+		return $this->dry()?FALSE:$this->get('csrf');
+	}
+
 	/**
 	*	Return IP address associated with specified session ID
 	*	@return string|FALSE
@@ -123,10 +146,16 @@ class Session extends Mapper {
 	**/
 	function __construct(\DB\SQL $db,$table='sessions') {
 		$db->exec(
-			'CREATE TABLE IF NOT EXISTS '.
-				(($name=$db->name())?($name.'.'):'').$table.' ('.
+			(preg_match('/mssql|sqlsrv|sybase/',$db->driver())?
+				('IF NOT EXISTS (SELECT * FROM sysobjects WHERE '.
+					'name='.$db->quote($table).' AND xtype=\'U\') '.
+					'CREATE TABLE dbo.'):
+				('CREATE TABLE IF NOT EXISTS '.
+					(($name=$db->name())?($name.'.'):''))).
+			$table.' ('.
 				'session_id VARCHAR(40),'.
 				'data TEXT,'.
+				'csrf TEXT,'.
 				'ip VARCHAR(40),'.
 				'agent VARCHAR(255),'.
 				'stamp INTEGER,'.
@@ -143,6 +172,30 @@ class Session extends Mapper {
 			array($this,'cleanup')
 		);
 		register_shutdown_function('session_commit');
+		@session_start();
+		$fw=\Base::instance();
+		$headers=$fw->get('HEADERS');
+		if (($csrf=$this->csrf()) &&
+			((!isset($_COOKIE['_']) || $_COOKIE['_']!=$csrf) ||
+			($ip=$this->ip()) && $ip!=$fw->get('IP') ||
+			($agent=$this->agent()) && !isset($headers['User-Agent']) ||
+				$agent!=$headers['User-Agent'])) {
+			$jar=$fw->get('JAR');
+			$jar['expire']=strtotime('-1 year');
+			call_user_func_array('setcookie',
+				array_merge(array('_',''),$jar));
+			unset($_COOKIE['_']);
+			session_destroy();
+			\Base::instance()->error(403);
+		}
+		$csrf=$fw->hash($fw->get('ROOT').$fw->get('BASE')).'.'.
+			$fw->hash(mt_rand());
+		if ($this->load(array('session_id=?',session_id()))) {
+			$this->set('csrf',$csrf);
+			$this->save();
+			call_user_func_array('setcookie',
+				array('_',$csrf)+$fw->get('JAR'));
+		}
 	}
 
 }
