@@ -52,7 +52,7 @@ abstract class Magic implements ArrayAccess {
 	**/
 	private function visible($key) {
 		if (property_exists($this,$key)) {
-			$ref=new \ReflectionProperty(get_class($this),$key);
+			$ref=new ReflectionProperty(get_class($this),$key);
 			$out=$ref->ispublic();
 			unset($ref);
 			return $out;
