@@ -179,7 +179,7 @@ var selfoss = {
                 else if (textStatus == "parsererror")
                     location.reload();
                 else if (errorThrown)
-                    alert('Load list error: ' + errorThrown);
+                    selfoss.showError('Load list error: '+errorThrown);
             }
         });
     },
@@ -202,7 +202,7 @@ var selfoss = {
                 selfoss.events.navigation();
             },
             error: function(jqXHR, textStatus, errorThrown) {
-                alert('Load tags error: '+errorThrown);
+                selfoss.showError('Load tags error: '+errorThrown);
             },
             complete: function(jqXHR, textStatus) {
                 $('#nav-tags').removeClass('loading');
@@ -260,6 +260,28 @@ var selfoss = {
                 }
             });
         }
+    },
+    
+    
+    /**
+     * anonymize links
+     *
+     * @return void
+     * @param parent element
+     */
+    showError: function(message) {
+        if(typeof(message) == 'undefined') {
+            var message = "Oops! Something went wrong";
+        }
+        var error = $('#error');
+        error.html(message);
+        error.show();
+        window.setTimeout(function() {
+            error.click();
+        }, 10000);
+        error.unbind('click').click(function() {
+            error.fadeOut();
+        });
     }
 };
 
